"""
load string representation and convert 'tags'-value into ChangedList:
=====================================================================

>>> _json = _JSON({"tags": (ChangedList,)})
>>> r = _json.loads('{"foo": [1, 2], "tags": [1, 2, 3, 4]}')
>>> r["tags"]
[1, 2, 3, 4]
>>> r["tags"].changed
False
>>> r["tags"].append(45)
>>> r["tags"].changed
True
>>> isinstance(r["foo"], ChangedList)
False

dump object into string representation
======================================

>>> simplejson.dumps(r)
'{"foo": [1, 2], "tags": [1, 2, 3, 4, 45]}'

convert 'tags'-value into ChangedList:
======================================

>>> r = _json.convert_dict({"foo": [1, 2], "tags": [1, 2, 3, 4]})
>>> isinstance(r["tags"], ChangedList)
True

create a namespace related rule for conversion
==============================================

>>> _json = _JSON({"tags": (ChangedList, "first_type")})
>>> r = _json.loads('{"foo": [1, 2], "tags": [1, 2, 3, 4]}', "first_type")
>>> isinstance(r["tags"], ChangedList)
True
>>> r = _json.loads('{"foo": [1, 2], "tags": [1, 2, 3, 4]}', "second_type")
>>> isinstance(r["tags"], ChangedList)
False
"""

import simplejson

class ChangedList(list):
    
    def __init__(self, lst):
        """
        >>> l = ChangedList([1, 2, 3, 4])
        >>> l.changed
        False
        """
        list.__init__(self, lst)
        self.__changed = False
        
    @property
    def changed(self):
        return self.__changed
        
    def append(self, value):
        """
        >>> l = ChangedList([1, 2, 3, 4])
        >>> l.append(5)
        >>> l.changed
        True
        """
        list.append(self, value)
        self.__changed = True
        
    def extend(self, seq):
        """
        >>> l = ChangedList([1, 2, 3, 4])
        >>> l.extend([5,6])
        >>> l.changed
        True
        """
        list.extend(self, seq)
        self.__changed = True
        
    def insert(self, idx, value):
        """
        >>> l = ChangedList([1, 2, 3, 4])
        >>> l.insert(2, 5)
        >>> l.changed
        True
        """
        list.insert(self, idx, value)
        self.__changed = True
        
    def remove(self, value):
        """
        >>> l = ChangedList([1, 2, 3, 4])
        >>> l.remove(2)
        >>> l.changed
        True
        """
        list.remove(self, value)
        self.__changed = True
        
    def pop(self, idx=None):
        """
        >>> l = ChangedList([1, 2, 3, 4])
        >>> l.pop()
        4
        >>> l.changed
        True
        >>> l = ChangedList([1, 2, 3, 4])
        >>> l.pop(1)
        2
        >>> l.changed
        True
        """
        if idx is None:
            x = list.pop(self)
        else:
            x = list.pop(self, idx)
        self.__changed = True
        return x
        
class map_objects(object):
    def __init__(self, dct_map=None):
        self.__dct_map = dct_map
        
    def __call__(self, dct, type_url=None):
        if self.__dct_map is None:
            return dct
        for key, value in self.__dct_map.iteritems():
            try:
                value, type_cond = value
            except:
                value = value[0]
                type_cond = None
            if not (type_cond is None) and not (type_cond == type_url):
                continue
            try:
                dct[key] = value(dct[key])
            except KeyError:
                continue
        return dct
        
    def iterattributes(self):
        return self.__dct_map.iterkeys()
        
    
class _JSON(object):
    def __init__(self, obj_map):
        self.__obj_map = map_objects(obj_map)
        
    def loads(self, s, type_url=None):
        return simplejson.loads(s, object_hook=lambda x: self.__obj_map(x, type_url))
            
    def convert_dict(self, dct, type_url=None):
        return self.__obj_map(dct, type_url)
        
    def iterattributes(self):
        return self.__obj_map.iterattributes()
        
        
    
if __name__ == "__main__":
    import doctest
    doctest.testmod()
